// Metric: 0 - Chars and 6 - Pixels only

Class ClockField( parentObj ) of ENTRYFIELD( parentObj ) custom
/*

   Clock.cc
   Author: Gary White

   Version history: 
      3/22/98 original version by Gary White
      9/22/98 added a popup menu to allow the user to choose 12/24
      		  hour format and to choose to dislay or not display
              seconds.  This idea and most of the work on it was
              thanks to Daniel Howard.


   Purpose: Implements a simple clock that may be dropped on a form 
   in the form designer.  Typical implementation is (from the 
   command window, type

      set procedure to clock.cc additive

   then design your form as usual.  The clock will be on the 
   custom tab of the component palette.  Just drag it onto your form.

   Configurable properties:
   All the Font, Position, and visual properties are available in 
   the inspector while in the Form Designer.  Additional optional 
   items may be modified in the form's OnOpen event and are as 
   follows:

      timeZoneOffset 	integer (valid range -12 to + 12)
         If you want to display a time zone other than local, use 
         the form's OnOpen event to set clock.timeZoneOffset to 
         the desired time zone.  The offset value is based upon 
         Hours relative to GMT, i.e. US eastern standard time 
         is GMT-5; US central standard time is GMT-6, etc.

         Default value is local time

      TimeFormat			integer (valid range 0 or 1) 
         TimeFormat is used to select 12 hour or 24 hour displays.  
         Setting TimeFormat to 0 displays a 24 hour clock, 
         while setting TimeFormat to 1 displays a 12 hour clock 
         with an am/pm suffix.

         Default value is 1 (12 hour)

      ShowSeconds			boolean (true or false)
         ShowSeconds controls whether the clock displays the 
         seconds of the time.  When ShowSeconds is true, time 
         is display with hours, minutes and seconds.  When 
         ShowSeconds is false, only the hours and minutes
         are shown.

         Default value is false


   The current date in the time zone is shown as a speedtip.  
   This date display respects the setting of SET DATE.

*/

	// Hide properties that aren't relevant and protect the onOpen and 
   // when events.
	protect tabStop, enabled, dataLink, function, maxLength, ;
	   picture, validErrorMsg, validRequired, helpFile, helpID, ;
   	statusMessage,selectAll, memoEditor, phoneticLink, ;
	   colorHighLight, onOpen, when

   with (this)
      onOpen = class::clock_onOpen
      when = {;return false}
      height = 0.8636
      left = 2
      onRightMouseDown = class::clock_onRightMouseDown
      top = 1.5
      width = 10.7143
//      metric = 0	// Chars
      colorNormal = "BtnText/BtnFace"
      tabStop = false
      value = time()
      mousePointer = 1	// Arrow
      borderStyle = 1	// Raised
   endwith

   function onDesignOpen( lFromPalette )
      if not lFromPalette
         return
      endif
      if form.metric # 0 and form.metric # 6
         msgbox( "Please use either form metric of 0 (Chars) or "+;
                 "6 (Pixels) for this control." )
         return
      endif
      if form.Metric == 6 // Pixels
         this.height := Chars2Pixels(this.height,"Y" )
         this.width  := Chars2Pixels(this.width, "X" )
      endif
   return

   function clock_onOpen
   	this.value = ""	// hide display until we make calculations

      if type("this.ShowSeconds") == "U"	// default to not showing seconds
      	this.ShowSeconds = false
      endif

		if type("this.TimeFormat") == "U"	// default to 12 hour clock
         this.TimeFormat = 1
      endif

      this.dateTime = new date()	// create a date object to work with

      // Windows uses a GMT offset in minutes, but hours are easier
      // for people to work with, so we convert.
      this.localGMTOffset = this.dateTime.GetTimeZoneOffset() / 60

		// If a time zone offset has not been declared, use local time
      if type("this.timeZoneOffset") == "U"
      	this.timeZoneOffset = this.dateTime.getTimeZoneOffset() / 60
      endif

		// We store references to installed clocks in an array so we can 
      // clean up when closing.  First, check to see if the array exists.
   	if type("form.clocks") # "A"
      	// if not, create it
         form.clocks = new array()
      endif

		// Hang a reference to the clock on the form. If the clock
      // winds up being inside another container we'll still be 
      // able to find it.
    	form.clocks.add(this)

      //  Now create the timer as a property of the clock.
      this.Timer = new timer()

      //  Hang a reference to the clock (entryfield) on the timer object
      this.Timer.parent = this

      // Set the time interval to fire once per second
      this.Timer.interval = 1

		// Now tell the timer what to do.  Since the timer's parent is the
      // entryfield, all the timer has to do is set the value to the time
      // and update the speedtip to show the date.  Note that in this line
      // "this" refers to the timer object because that's who is calling it.
      this.Timer.OnTimer = {;this.parent.value=class::FormatTime(this.parent.TimeFormat)}

      // Now let's start the timer
      this.Timer.Enabled = true

      // Finally let's take care of cleaning up. The timer won't be disabled
      // automatically, so we sneak into the form's OnClose event.
      // First check to see if another clock has already done this
		if type("form.ClockInstalled") # "C"

      	// If not, set a flag so we know it's been done
      	form.ClockInstalled = "True"

         // next we test to see if something's already there.
	      if empty(form.onClose)

   	   	// if not, we just set it to disable the timer
      		form.onClose = {;class::TurnOffClocks()}
	      else
   	   	// otherwise, save the old OnClose
				form.sOnClose = form.onClose
         	// and construct a new one that disables the timer then calls the old one.
	         form.onClose = {;class::TurnOffClocks();form.sOnClose()}
   	   endif
      endif
      return
		
	function CurrentTime( nTimeZoneOffset )
   	// nTimeZoneOffset is in hours relative to GMT because that's 
      // easier for people to work with and remember.  However, 
      // Windows calculates the offset in minutes, so we need to 
      // convert.  Returns a date object.
      local nRealOffset, d
      nRealOffset = ( nTimeZoneOffset) * 60
      d = new date()

      // convert to GMT
      d.setMinutes( d.getMinutes() - d.getTimeZoneOffset())

      d.setMinutes( d.getMinutes() + this.parent.timeZoneOffset * 60 )
   	return d

   function FormatTime( nTimeFormat )
   	// While this is a method of the Clock, when it is called, it is called
      // by the timer object.  Inside this method "THIS" refers to the timer.
   	local cTimeString, d
      static nHour
      d = class::CurrentTime( this.parent.timeZoneOffset )
		do case
      	case nTimeFormat = 0	// 24 hour
         	cTimeString = transform( d.getHours(), "@L 99") +;
            	":" + transform( d.getMinutes(), "@L 99") +;
               iif(this.parent.ShowSeconds = true, ":" + transform( d.getSeconds(), "@L 99"),"")
         case nTimeFormat = 1 // 12 hour
         	do case
            	case d.getHours() < 12	// morning
		         	cTimeString = transform( d.getHours(), "99") +;
      	      		":" + transform( d.getMinutes(), "@L 99") +;
		               iif(this.parent.ShowSeconds = true, ":" + +;
                     transform( d.getSeconds(), "@L 99"),"") +" am"
               case d.getHours() > 12	// afternoon
               	cTimeString = transform( d.getHours() - 12, "99") +;
      	      		":" + transform( d.getMinutes(), "@L 99") +;
		               iif(this.parent.ShowSeconds = true, ":" + +;
                     transform( d.getSeconds(), "@L 99"),"") +" pm"
					otherwise					// noon
		         	cTimeString = transform( d.getHours(), "99") +;
      	      		":" + transform( d.getMinutes(), "@L 99") +;
		               iif(this.parent.ShowSeconds = true, ":" + +;
                     transform( d.getSeconds(), "@L 99"),"") +" pm"
            endcase
      endcase
		this.parent.speedtip=dtoc(d)
      release object d
      return cTimeString

	procedure TurnOffClocks
   	local i, oClock
      for i=1 to form.clocks.size
      	oClock = form.clocks[i]
         oClock.Timer.enabled = false
         release object oClock.Timer
         oClock.Timer = null
      endfor

   function clock_onRightMouseDown(flags, col, row)
   	local nTop, nLeft
      store 0 to nTop, nLeft

      if type("form.ClockPop") == "U"
         form.ClockPop = New ClockPopUp(form, "xx")
      endif

      // Here we have to do a little math to compensate for 
      // differences in the form.metric.  The popup will use
      // Chars so we need to convert to units that will place
      // it where we want it.
      do case
      	case form.metric == 0	// chars
            nTop := ( this.top + row ) + 1
            nLeft := this.left + this.width + col
      	case form.metric == 1	// twips
            nTop := ( this.top + row ) / 330 + 1
            nLeft := ( this.left + this.width + col) / 105
      	case form.metric == 2	// points
            nTop := ( this.top + row ) / 16.5 + 1
            nLeft := ( this.left + this.width + col) / 5.25
      	case form.metric == 3	// Inches
            nTop := ( this.top + row ) / 0.229 + 1
            nLeft := ( this.left + this.width + col) / 0.073
      	case form.metric == 4	// centimeters
            nTop := ( this.top + row ) / 0.582 + 1
            nLeft := ( this.left + this.width + col) / 0.18525
      	case form.metric == 5	// millimeters
            nTop := ( this.top + row ) / 5.82 + 1
            nLeft := ( this.left + this.width + col) / 1.8525
      	case form.metric == 6	// pixels
            nTop := ( this.top + row ) / 22 + 1
            nLeft := ( this.left + this.width + col) / 7
      endcase

      // now tell the popup where to display
      form.ClockPop.top = nTop
      form.ClockPop.left = nLeft

      // hang a reference to this clock
		form.CurrentClock = this

      // and finally open the popup menu
      form.ClockPop.Open()
      return
endclass
// End of Class: CLOCK

** END HEADER -- do not remove this line
//
// Generated on 08/18/98
//
parameter formObj, popupName
new clockPOPUP(formObj, popupName)

class clockPOPUP(formObj, popupName) of POPUP(formObj, popupName)

   with (this)
      onInitMenu = class::ROOT_ONINITMENU
   endwith

   this.showSeconds = new menu(this)
   with (this.showSeconds)
      onClick = class::showSeconds_onClick
      text = "Show Seconds"
   endwith

   this.format12hr = new menu(this)
   with (this.format12hr)
      checked = true
      onClick = class::format12hr_onClick
      text = "Time in 12/hr Format"
   endwith

   function root_onInitMenu
      this.format12hr.checked = form.CurrentClock.TimeFormat == 1
      this.showSeconds.checked = form.CurrentClock.ShowSeconds
      return

   function format12hr_onClick
   	this.checked = not this.checked
      form.CurrentClock.TimeFormat = iif( this.checked, 1, 0 )
   	return

   function SHOWSECONDS_onClick
   	this.checked := not this.checked
      form.CurrentClock.ShowSeconds = this.Checked
	   return
endclass

// Convert Char dimensions to Pixel -- code by
// Rich Muller (Autotraker):
*************************************************
function Chars2Pixels(Chars,cType)
*************************************************
   local YFactor, XFactor, YInPixels, XInPixels
   YFactor = .0455 // Vertical Factor
   XFactor = .1429 // Horizontal Factor
   YInPixels = Chars / YFactor
   XInPixels = Chars / XFactor
return iif(cType="X",XInPixels,YInPixels)

