/*
 File: Clipb.Cc
 By Peter Rorlick
 April 14, 1999
 Written for: Visual dBASE 7.01

 Purpose:
  The programmatic transfer of text to and from the Windows 
  clipboard and VdB7. The class works only with text.

 This version of the TextClipboard class is simpler and 
 faster than the one in Clipbord.cc that was written by 
 Bowen Moursund.
 This version does NOT require dbmem.dll
 Note that the methods here have the same names as in Bowen's version.
 Note the addition of the Close() method.

 Example of use:
   set procedure to Clipb.CC additive
   _app.Clip = new TextClipboard()
   _app.Clip.SetClipboardText("Did it work?")

 Problem: If you are working with an entryfield of
          more than 5000 characters, the technique
          used here will only select the first 5000
          characters ...
*/

CLASS TextClipboard

   protect Text
   this.Text = ""

   this.f = new form()
   this.f.visible = false    // User will never see this form.
   this.e = new entryfield(this.f)  // For cutting
   this.ed = new editor(this.f)  // A receptacle for pasting.
   this.f.open()


   //  Use this function to set this.Text programmatically
   FUNCTION SetText
   	// Receive parm as private for type() check
   	parameter cStr

      local lReturn
      lReturn = false

      // Nothing but strings, thank you
      if type( "cStr" ) == "C"
      	this.Text = cStr
         lReturn = true
      endif

   RETURN lReturn


   //  Use this function to retrieve the contents of this.Text
	FUNCTION GetText
   	RETURN this.Text      		


   // Store the contents of the Windows clipboard to this.Text
   FUNCTION GetClipboardText
      this.ed.value := ''
      this.ed.paste()
      this.Text = this.ed.value
   RETURN


   // Store the contents of this.Text to the Windows clipboard
   // Optionally pass a string directly to this method
   FUNCTION SetClipboardText( cStr )

   	//  Did we receive a parameter?
   	if argCount() > 0 and not class::SetText( cStr )
      	RETURN
      endif

      // Any text to copy?
      if len( this.Text ) < 1
        RETURN
      endif

      this.e.value := this.text
      this.e.keyboard( '{Home}'+'{Shift+End}' )
      this.e.cut()

   RETURN


   FUNCTION Close
      this.f.close()
   RETURN

ENDCLASS

// EoF: Clipb.Cc