function DeleteTree
	parameters startDir
   #Include Windef.h
   /* Rich - www.autotraker.com
      DeleteTree(getEnv("TEMP"))
      this routine uses getDir.prg from the dUFLP.
      We we the folders in our path using getDir
      Then walk though the folders backwards because
      we need the deepest back to the start dir.
      We attempt first to set the attibute of the file
      to normal to ease deletion, then we delete it.
      We next delete the folder the files were in. 
      Because of XP sometines making folders read only
      we also attempt to make the folders attibute normal
      before deleting it. The API calls I'm using are
      RemoveDirectory since dBASE doesn't have this 
      function build in, SetFileAttributes so I attempt
      to remove read only, hidden, or system attibutes. 
      And DeleteFile since it is capable of deleting files
      that the dBASE file object can't handle. 
   */

   if empty(startDir)
   	msgbox("You must have a starting folder")
      return
   endif

   // check backslash
   startDir = iif(startDir.right(1)=="\",startDir.left(startDir.length-1),startDir)

   // get directories in path
   set procedure to :dUFLP:getDirs.prg additive
   a = getDirs(startDir)
   close procedure :dUFLP:getDirs.prg

   // walk though directories backwards
   for i = a.size to 1 step -1

      // build full path from array
      cDir = startDir + a[i]

      aFiles = new Array()
      nFiles = aFiles.dir(cDir+"\*.*", "HS")
      for nFile = 1 to nFiles
         cFile = cDir+"\"+aFiles[ nFile, 1 ]
         ? "Delete File: " + cFile
         SetAttribNormal(cFile)
         DeleteFile(cFile)
      endfor

      ? "Remove Folder: " + cDir
      SetAttribNormal(cDir)
      RemoveDir(cDir)

   endfor

   // we bypassed files in the start dir, so handle them now
   aFiles = new Array()
   nFiles = aFiles.dir(startDir+"\*.*", "HS")
   for nFile = 1 to nFiles
      cFile = startDir+"\"+aFiles[ nFile, 1 ]
      ? "Delete File: " + cFile
      SetAttribNormal(cFile)
      DeleteFile(cFile)
   endfor

return


**************************************************
function DeleteFile(lpFileName)
**************************************************
   /* Rich - www.autotraker.com
      deletes a file
      usage DeleteFile("c:\temp\dummy.txt")
      returns sucess
   */
   if type("DeleteFileA") # "FP"
      extern BOOL DeleteFileA(;
          LPCTSTR; //lpFileName - pointer to name of file to delete  
          ) kernel32 from "DeleteFileA"	
   endif
return DeleteFileA(lpFileName)

**************************************************
function SetAttribNormal(FileName)
**************************************************
   /* Rich - www.autotraker.com
      Sets all Attributes of a file or folder to normal
      usage SetAttribNormal("c:\temp\dummy.txt")
      usage SetAttribNormal("c:\temp")
      returns sucess
   */
   if type( "SetFileAttributes" ) # "FP"
      extern cLogical SetFileAttributes( cString, cInt );
             kernel32 from "SetFileAttributesA"
   endif
return SetFileAttributes(FileName, 0x00000080 )

**************************************************
function RemoveDir(cDir)
**************************************************
   /* Rich - www.autotraker.com
      Removes a empty folder
      usage RemoveDir("c:\temp")
      returns sucess
   */
   if type("RemoveDirectory") # "FP"
     extern cLogical RemoveDirectory( cString ) kernel32;
     from "RemoveDirectoryA"
   endif
   lSuccess = false
   try
      lSuccess = RemoveDirectory(cDir)
   catch (exception e)
   endtry
return lSuccess
