/*
   --------------------------------------------------------------------
      HTMLLinkText.cc
      Ken Mayer
      September, 2005

      This is a basic Text control that has been modified
      to act as an HTML Hypertext Link. It uses the API
      function below called "OpenURL" from the dUFLP's
      MiscAPI.prg to actually execute the URL.

      Special Thanks to J.P. Martel for pointing out
      I was doing too much work with the string object's
      link() method (which was killing color changes,
      and not necessary at all for the purpose of this
      control).

      The onMouseOver/onMouseOut events will only be set
      if the user is working with dBASE Plus 2.6 or later,
      as there were problems on some operating systems
      in 2.5 with the implementation of these events. This
      was fixed in 2.6.

      Usage:
         set procedure to HTMLLinkText.cc additive
           * Create a form, or open an existing form
             in the form designer, place an instance
             of this control on the form.
           * Set the text property in the inspector.
           * In an OnOpen event, or form Init() (custom
             event), you can set these properties:

            url = The URL you wish to execute. Be
                  sure to include the proper
                  beginning. For example:
                     "http://www.goldenstag.net/dbase"
                  or
                     "mailto:ken@goldenstag.net"
                  This is required.
                     Note: this should be able to use any
                     valid URL, including as noted
                       http://someurl
                       mailto:some_email@address.com
                         but also:
                       somelocal.htm
                       ftp:someFTPsite
                         // etc.

                  It should be noted that for a "mailto:" 
                  reference, you can add options for
                  subject, even a default "body" for
                  the email. This will not send an email
                  message, but will, rather, bring up the
                  default email software, set the address
                  and if you use the options for subject
                  and/or body, it will insert those, but
                  it will not send the email until the user
                  elects to send it. The following are some
                  examples for using subject and body:

                     mailto:Myaddress.com?subject=A Subject
                     mailto:Myaddress.com?body=Message Body
                     mailto:Myaddress.com?subject=A Subject&Body=Message Body

                  Note that when using both, you use a ? for the separator
                  between the address and the subject, and a & for
                  the separator between the subject and the body ...

                  Using the message body can be a bit tricky, but 
                  it can be done ... 
           --------------------
           Optional Properties:
            NormalBGColor = standard color string
                  This is the background color
                  of the control. By default it will use
                  the parent object's colorNormal property.

            NormalFGColor = standard color string
                  This is the color you wish
                  the text to normally display.
                  The default is blue.

            HighlightBGColor = standard color string
                  This is the color to display
                  for the background of the text
                  when the mouse is over it.
                  By default, this will be the
                  color of the parent of the
                  control. 

            HighlighFGColor = standard color string
                  The color the text should display
                  when the mouse is over the control.
                  The default is red.

            MouseDownBGColor = standard color string
            MouseDownFGColor = standard color string
                   The color of the text when the
                   mouse button is 'down' ... 
         
         Once done setting any properties, call the
         control's Setup method:
          function form_onOpen
             form.HTMLLINKTEXT1.NormalFGColor    := "purple"
             form.HTMLLINKTEXT1.HighlightFGColor := "red"
             form.HTMLLINKTEXT1.HighlightBGColor := "white"
             form.HTMLLINKTEXT1.MouseDownFGColor := "red"
             form.HTMLLINKTEXT1.URL              := "http://www.goldenstag.net/dbase"
             form.HTMLLinkText1.setup()
          return
   --------------------------------------------------------------------
*/ 
class HTMLLinkText( oParent ) of Text( oParent ) custom
   protect colorNormal, NormalColor, MouseOverColor, MouseDownColor

   // stock properties/events:
   this.text              := "HTMLLinkText Control"
   this.fontUnderLine     := true
   if __version__ => 2.6
      this.onMouseOver    := class::MouseOver
      this.onMouseOut     := class::MouseOut
   endif
   this.onLeftMouseDown := class::LeftMouseDown
   this.onLeftMouseUp   := class::LeftMouseUp
   if smallFonts()
      this.fontSize := 10
   else
      this.fontSize := 8
   endif
   this.height := 0.73
   this.width := 19.14

   // properties for foreground color of text:
   this.NormalFGColor    = "blue"   // Normal
   this.MouseDownFGColor = "purple" // Mouse down
   this.HighlightFGColor = "red"    // MouseOver ...

   // Background color of text:
   // minor situation -- it may be possible that a form
   // or parent container control has the colorNormal
   // property set with foreground/background -- not
   // likely, but possible. In that unlikely case, we
   // will have a slash in the string, and need to
   // parse it out:
   local cCN, cBackClr
   cCN = this.parent.colorNormal
   if "/" $ cCN
      cBackClr = right( cCN, at( "/", cCN )-1 )
   else
      cBackClr = cCN
   endif
   // default custom background colors:
   this.NormalBGColor    = cBackClr
   this.HighlightBGColor = cBackClr
   this.MouseDownBGColor = cBackClr
   
   // The URL to execute ...
   this.URL              = "" // Developer must define this

   class::Setup()
   // end of constructor code
   // -----------------------------------------------------

   function Setup
      // Call this method from the form's onOpen or other 
      // starting event ...

      // Set default color ...
      this.colorNormal     := this.NormalFGColor+"/"+this.NormalBGColor
      // custom color settings:
      this.NormalColor      = this.colorNormal
      this.MouseOverColor   = this.HighlightFGColor+"/"+this.HighlightBGColor
      this.MouseDownColor   = this.MouseDownFGColor+"/"+this.MouseDownBGColor
   return

   function onDesignOpen( lFromPalette )
      if not lFromPalette
         return
      endif
      if form.metric # 0 and form.metric # 6
         msgbox( "Please use either form metric of 0 (Chars) or "+;
                 "6 (Pixels) for this control." )
         return
      endif
      if form.Metric == 6 // Pixels
         this.height := Chars2Pixels(this.height,"Y" )
         this.width  := Chars2Pixels(this.width, "X" )
      endif
   return


   function LeftMouseDown
      // set the color, but don't do anything else:
      this.colorNormal := this.MouseDownColor
   return

   function LeftMouseUp
      // set the color appropriately -- if
      // the version of dBASE is 2.6 or later,
      // set the MouseOver color, otherwise,
      // set the "normal" color:
      if __version__ => 2.6
         class::MouseOver()
      else
         this.colorNormal := this.NormalColor
      endif
      // don't forget to execute the hypertext link:
      class::CallURL()
   return

   function MouseOver
      // When the mouse is over this control,
      // change the color so the user sees
      // a difference ...
      this.colorNormal  := this.MouseOverColor
      this.mousePointer := 13 // Hand
   return

   function MouseOut
      // When we leave this control, change
      // the color back to "normal" ...
      this.colorNormal  := this.NormalColor
      this.mousePointer := 1 // Arrow
   return

   function CallURL
      // Actually call the URL -- or at least try to. 
      // This should work for standard URLs, including
      //     http://someurl
      //     localwebpage.htm
      //     ftp:someurl
      //     mailto:some_email@address
      OpenURL( this.URL )      
   return
   
endclass

* ---------------------------------------------------------------- 
*      OpenURL -- opens a browser with the URL you pass to it.
*      Posted by Romain Strieff on the VdBASE Newsgroups
*      Updated January 12, 1999
*      This version should work for both 5.x and 7.x
*          (which is why the old-style comments)
*      Usage:
*        OpenURL( "someurl" )
*      Example:
*        OpenURL( "http://www.dbase.com" )
*           or
*        OpenURL( "myhtml.htm" )
*  ---------------------------------------------------------------- 
function OpenURL( cURL )
   #define SW_SHOWNORMAL     1
   #define SW_SHOWMINIZED    2
   #define SW_SHOWMAXIMIZED  3
   #define SW_SHOWNOACTIVATE 4

   #define WINAPI_A "A"
   #define WINAPI_SHELL "SHELL32"

   if type( "ShellExecute" ) # "FP"
      extern cHandle ShellExecute( cHandle, cString, cString, ;
             cString, cString, cINT ) WINAPI_SHELL ;
             from "ShellExecute"+WINAPI_A
   endif

   local nHandle
   private sd         && macro needs private not local
   sd = set( "dire" ) && sometimes it *could* change dirs, so 
                      && we are preventing it ...
   nHandle = ShellExecute( _app.framewin.hwnd, "open", cURL, ;
                          null, null, SW_SHOWNORMAL )
   * if the directory changed:
   if sd # set("dire")
      set dire to "&sd."
   endif
return nHandle
*-- EoF: openURL()

// Convert Char dimensions to Pixel -- code by
// J.P. Martel:
Function Chars2Pixels(OldSize, cType)
 return(int(OldSize / iif(cType=="X", 0.1429, 0.0455)))

Function Smallfonts
/*
   Programmer   : Romain Strieff [dBVIPS]
   Last Modified: 04/20/1999
   Rev. History.: 04/20/1999 -- Original
   Notes........: Returns True if Smallfonts are used
   .............: in Windows menus
   Written for..: Visual dBASE (32-bit versions)
   Usage .......: ? SmallFonts()
*/
   local dc,result
   extern CINT        GetDeviceCaps(cHandle, CINT) gdi32
   extern CINT        ReleaseDC(cHandle, cHandle) user32
   extern cHandle     GetDC(cHandle) user32
   #define LOGPIXELSX    88
   dc=getdc(0)
   result=GetDeviceCaps(dc,LOGPIXELSX)
   ReleaseDc(0,dc)
return result=96
// eof: SmallFonts()

