/*
   ------------------------------------------------------------------
   Num2Word.CC
   Tom Arnold
   03/30/1998

  NumToWrd is a Custom Control to automatically
  print the words that express a number typed into
  an Entryfield, as you would want to do for the 
  Amount of a Check.

  Numeric numbers typed into NumToWrd will be converted
  to words and displayed in the second entryfield
   (which is disabled).

   Modifications: 
      04/12/1998 -- Ken Mayer -- changed to a container -- the
        original only had the one entryfield, this has both
        as required. If you do not want a border, change the
        borderStyle property on the container to 3 (None).

   Usage:
      set procedure to num2word.cc additive
      // in the form designer, drag this control
      // onto a form from the component palette
      // move the entryfields around as needed, you may 
      // need to widen the container's edges as well.


  There are three custom properties
 			this.NUMENTRYFIELD.fieldLen
           FieldLen sets the maximum length of the line of 
           words in the display entryfield. It operates 
           by inserting enough spacers to keep the line 
           of words more or less constant. This has mixed 
           results because of the variable length of 
           characters. I find that the Courier font 
           works pretty good.
 			this.spacer
          Spacer sets the Char value of the Character 
           used to separate the dollars and the cents.
 			this.NUMENTRYFIELD.dollarLimit
          DollarLimit sets the maximum amount that can be 
           entered into numToWrd.The default limit is 
           10000000, which permits up to $9,999,999.00. 
           1000000 permits up to $999,999.00, and 100000, 
           permits up to $99,999.00 .
   ------------------------------------------------------------------
*/

class NUM2WORDS( oFormObj ) of CONTAINER( oFormObj ) custom

   with ( this )
      transparent = true
      left = 0.8571
      top = 1.0455
      width = 90
      height = 3.8182
      borderStyle = 10	// Etched Out
   endwith
   // custom property of control
   this.spacer = "-"

   this.NUMENTRYFIELD = new ENTRYFIELD( this )
   with( this.NUMENTRYFIELD )
      top = 1
      left = 1
      function="J999999.99"
      fontName = "Courier"
      fontSize = 10
      value = ""
      onLostFocus = CLASS::NUMENTRY_OnLostFocus
      Key = CLASS::NUMENTRY_Key
      width = 15
   endwith
   // custom properties of NUMENTRYFIELD 
   this.NUMENTRYFIELD.fieldLen = 60
   this.NUMENTRYFIELD.dollarLimit = 10000000

   this.WORDENTRYFIELD = new ENTRYFIELD( this )
   with( this.WORDENTRYFIELD )
      when = {; return false } // disable it
      top = 2
      left = 1
      width = 87
      fontName = "Courier"
      fontSize = 10
      value = ""
   endwith

   function NUMENTRY_OnLostFocus
      if at( '.', this.value ) > 0
         this.value = ;
              substr(this.value,1,at('.',this.value)-1) ;
              + left((substr(this.value,at('.',this.value))+"0"),3)
      else
         this.value+= ".00"
      endif

   function NUMENTRY_key( nChar, nPosition, bShift, bControl ) 
       if int(val(this.value))=>100000 and this.dollarLimit<=100000 // ..limit digits
          msgbox( "Maximum amount for a check is $99.999.99",;
                  "Check Limit", 16 + 1 ) 
          this.value = ""
          this.parent.WordEntryField.value = ""
          RETURN 0
       elseif int(val(this.value))=>1000000 and this.dollarLimit<=1000000 // ..limit digits
          msgbox( "Maximum amount for a check is $999.999.99",;
                  "Check Limit", 16 + 1 ) 
          this.value = ""
          this.parent.WordEntryField.value = ""
          RETURN 0
       elseif int(val(this.value))=>10000000 and this.dollarLimit<=10000000 // ..limit digits
          msgbox( "Maximum amount for a check is $9,999.999.99",;
                  "Check Limit", 16 + 1 ) 
          this.value = ""
          this.parent.WordEntryField.value = ""
          RETURN 0
       endif

       if nextKey() # 0
          // Wait until there are no keystrokes pending, and...
         RETURN (nChar # 255 ) // ...ignore keystroke if special key
       endif

       if nChar == 255
          if not empty( this.value )
             this.parent.PrintInWords( this.value )
             // ...permit two decimals only
             if len(substr(this.value+"X",at(".",this.value)))>3
                keyboard chr(9)
             endif
          endif
          nChar = 0
       else
         if nChar >= 48 .and. nChar =< 57
           // Stuff special key to refire key event
           keyboard chr(255)
         endif
       endif
    RETURN nChar

    function PrintInWords( cValue )
       LOCAL n, d, m, m2, dec
       n = val( cValue )
       dec = 100 * mod(n,1)
       if n > 0
          this.WordEntryField.value = ""
       endif

       // Assume n<10,000,000

       if int(n/1000000)>0 
          this.printOnes(int(n/1000000)) // Hundred thousands
          this.WordEntryField.value += " "+"Million"
          n=n%1000000
       endif

       if int(n/1000)>0
          if int(n/100000)>0
             this.printOnes(int(n/100000)) // Hundred thousands
             this.WordEntryField.value += " "+"Hundred"
          endif

          m=int(mod(n,100000)/1000)

          if m<10
             this.printOnes(m)
          elseif m<20 .and. m=>10
             this.printTeens(m)
          elseif m<100 .and. m=>20
             d=int(m/10)
             if d>0
                this.printTys(d)
             endif
             m=int(mod(m,10))
             if m>0
                this.printOnes(m)
             endif
          else
             msgbox(" Error ")
          endif
          this.WordEntryField.value += " "+"Thousand"
       endif

       n100=mod(n,1000)
       d=int(n100/100)
       if d>0
          this.printOnes(d)
          this.WordEntryField.value += " "+"Hundred"
       endif
       m=int(mod(n100,100))
       if m<10
          this.printOnes(m)
       elseif m<20 .and. m=>10
          this.printTeens(m)
       elseif m<100 .and. m=>20
          d=int(m/10)
          if d>0
             this.printTys(d)
          endif
          m=int(mod(m,10))
          if m>0
             this.printOnes(m)
          endif
       else
          msgbox(" Error ")
       endif
       fill=replicate( this.spacer, this.NUMENTRYFIELD.fieldLen + ;
                    10-len( trim( this.WordEntryField.value ) ) )
       if dec=0
          this.WordEntryField.value += fill+" and 00/00"
       else
          dec=str(dec,2,2,"0")
          this.WordEntryField.value += " and "+dec+"/00"
       endif

   function printTys(nChar)
      cChar=ltrim(str(nChar))

      DO CASE
         case cChar="2"
              this.WordEntryField.value += " Twenty"
         CASE cChar="3"
              this.WordEntryField.value += " Thirty"
         CASE cChar="4"
              this.WordEntryField.value += " Forty"
         CASE cChar="5"
              this.WordEntryField.value += " Fifty"
         CASE cChar="6"
              this.WordEntryField.value += " Sixty"
         CASE cChar="7"
              this.WordEntryField.value += " Seventy"
         CASE cChar="8"
              this.WordEntryField.value += " Eighty"
         CASE cChar="9"
              this.WordEntryField.value += " Ninety"
      ENDCASE 

   function printTeens(nChar)
      cChar=ltrim(str(nChar))
      DO CASE
         CASE cChar="10"
              this.WordEntryField.value += " Ten"
         CASE cChar="11"
              this.WordEntryField.value += " Eleven"
         CASE cChar="12"
              this.WordEntryField.value += " Twelve"
         CASE cChar="13"
              this.WordEntryField.value += " Thirteen"
         CASE cChar="14"
              this.WordEntryField.value += " Fourteen"
         CASE cChar="15"
              this.WordEntryField.value += " Fifteen"
         CASE cChar="16"
              this.WordEntryField.value += " Sixteen"
         CASE cChar="17"
              this.WordEntryField.value += " Seventeen"
         CASE cChar="18"
              this.WordEntryField.value += " Eighteen"
         CASE cChar="19"
              this.WordEntryField.value += " Nineteen"
      ENDCASE 

   function printOnes(nChar)
      cChar=ltrim(str(nChar))
      DO CASE
         CASE cChar="1"
              this.WordEntryField.value += " One"
         CASE cChar="2"
              this.WordEntryField.value += " Two"
         CASE cChar="3"
              this.WordEntryField.value += " Three"
         CASE cChar="4"
              this.WordEntryField.value += " Four"
         CASE cChar="5"
              this.WordEntryField.value += " Five"
         CASE cChar="6"
              this.WordEntryField.value += " Six"
         CASE cChar="7"
              this.WordEntryField.value += " Seven"
         CASE cChar="8"
              this.WordEntryField.value += " Eight"
         CASE cChar="9"
              this.WordEntryField.value +=  " Nine"
      ENDCASE 

endclass
// Num2Words.CC