// Metric: Should work with both Character and Pixels
/*
   ------------------------------------------------------------------
   Password.cc
   Author: Ken Mayer
   Date..: Spring, 1998
           August 21, 2001
           April 2, 2003
           January 27, 2004
           June 15, 2004 -- minor little fixes ...
           July 15, 2004 -- added captureEnter property, see below.

   A variation on the Password CC that shipped with Visual dBASE 5.x
   The original code is 1994 (c) Borland.

   August, 2001: Added caseSensitive and minLength properties,
      and code in isEnteredPasswordOK and LostFocus events
      to check for these values.

   April, 2003: Modified so that the IsEnteredPasswordOk event 
      increments the numtries and then compares against 
      numTriesAllowed, and if so, it sets the stopTrying property 
      to true. This allows a form to close if using canClose
      event, and so on. See TestPass.wfm 

      Check details in code, including LostFocus ...

   January, 2004: Added the CheckSelect API function prototype to
      detect if the user has selected any text and use this as a
      basis to determine the behavior of the Backspace and Delete
      keys.
      
      Modifications to this class done by Marc Hamelin.
      Original code from Marc VdB.

   April, 2005: Added captureEnter property and code in Procedure 'Key'
      to ignor or capture 'ENTER' key activity -- when captureEnter is True
      and PasswordEntry length is 0; an Alert message is generated the 
      same as XML Command LOGOUT's password behavior. -- Wes Rue

   Usage:
      set procedure to password.cc additive
      // in the form designer, place an instance
      // of the password entryfield onto the form
      // set the "correctPassword" to a value
      // in some fashion, you probably do not want 
      // to hardcode it, but ... when you know the
      // userid, get the password from the same
      // place, and assign it:
      form.passwordEntry1.setCorrectPassword( form.rowset.fields["password"].value )

      // set the number of retries allowed:
      form.passwordEntry1.numTriesAllowed = 3 // (default)

   ==========================================================
   NOTE: See detailed examples in TESTPASS.WFM and 
         CHNGPASS.WFM in the dUFLP library ...
   ==========================================================

   -- Custom Properties:
   maskChar -- character that appears instead of what's typed in
               Defaults to "*", but with some creativity, you 
               could change it to just about anything. You may
               want to consider using a dingbat font, for example,
               (setting the passwordEntryfield's fontName) and 
               using a character from that ... 
   numTriesAllowed -- defaults to 3, but can be changed 
               programmatically -- if you set it to 0,
               there won't be a check -- the user can keep 
               trying ...
   numTries -- do not try to change this -- it's the counter
               used to determine if the number of tries allowed
               has been reached.
   stopTrying -- logical value used to exit ... [defaults to false]
   validCheckRequired -- logical, defaults to true -- 
               used for such situations as a change password form
   caseSensitive -- defaults to false -- if it's false, then
               all tests compare against the uppercase version
               of the values, if it is true, then we do not,
               and a value of "TeSt" will not be equal to "Test",
               which will not be equal to "test", or "TEST".
   minLength -- minimum length of password -- defaults to zero,
               which means no error checking for length will
               occur. 
   captureEnter -- defaults to false -- if it is false, then 'ENTER'
   				key is ignored; if it is true, pressing 'ENTER' key
               when PasswordEntry length is 0 generates Alert message
               the same as XML Command LOGOUT's password behavior.

   // See Example: TestPass.wfm (which uses ChngPass.wfm)
   ------------------------------------------------------------------
*/

#define BACKSPACE_KEY     8     // -- ASCII value of Backspace key
#define DELETE_KEY        127   // -- ASCII value of Delete key
#define ENTER_KEY         13	  // -- ASCII value of Enter key

class PasswordEntry( oParent ) of Entryfield( oParent ) custom
   with( this )
      Border      := true
      PageNo      := 1
      Value       := ""
      MaxLength   := 15
      Height      := 1
      Width       := 15
      onOpen      := CLASS::ONOPEN
      Key         := CLASS::KEY
      onLostFocus := CLASS::LostFocus
   endwith
   // Add custom properties
   protect numTries
   this.maskChar           = "*"  // mask character 
   this.numTriesAllowed    = 3    // number of tries allowed
   this.numTries           = 0    // counter ...
   this.stopTrying         = false
   this.validCheckRequired = true
   this.caseSensitive      = false
   this.minLength          = 0  
   this.captureEnter       = false  

   function onDesignOpen( lFromPalette )
      if lFromPalette and form.metric == 6 // Pixels
         this.height :=  22
         this.width  := 105
      endif
   return

   // -------------------------------
   Procedure OnOpen(nChar, nPosition)
      // Protect properties
      protect correctPassword

      // Declare the CheckSelect API function prototype
      if Type("CheckSelect") # "FP"
         extern CLONG CheckSelect(CHANDLE, CUINT, ;
            CPTR CLONG, CPTR CLONG) user32 from "SendMessageA"
      endif

      // Custom properties
      if type("this.enteredPassword") = "U"
         this.enteredPassword = ""
         this.correctPassword = ""    // This property must be manually assigned
                                      // whereever this control is instantiated
      endif

      // custom _app property:
      _app.PasswordValid = false


   // -----------------------------------
   Procedure SetCorrectPassword(password)

     // Assigns value to this.correctPassword
      private passwordLen
      passwordLen          = len(password)
      this.maxLength       = passwordLen
      this.correctPassword = password
      this.enteredPassword = ""

   // -------------------------
   Function IsEnteredPasswordOk

      // Modified to deal with checking for case sensitivity
      private cPass
      cPass = this.enteredPassword
      cCorrect = this.correctPassword


      // check for case sensitivity -- Added KJM, August 21, 2001
      if NOT this.caseSensitive
         cPass = cPass.toUpperCase()
         cCorrect = this.correctPassword.toUpperCase()
         if cPass == cCorrect // if we match:
            _app.PasswordValid := true
         endif
      endif

      // so we can get out of the loop!
      this.numTries++  // increment counter
      if this.numTries => this.numTriesAllowed
         this.stopTrying := true
      endif

   return ( cPass == cCorrect )
  

   // ----------------------------
   Procedure Key(nChar, nPosition)

      // Handles keys entered in the password entryfield
      private enteredChar, returnValue

      // Get the character positions of the selected text (if any)
      local SelectStartPos, SelectEndPos
      SelectStartPos = 0
      SelectEndPos = 0
      CheckSelect(this.hwnd,0xB0,SelectStartPos,SelectEndPos)

      enteredChar = chr(nChar)
      returnValue = true                // By default output whatever key was typed
      do case                           // Check for keys that modify the value
         case nChar = BACKSPACE_KEY
            // if the text is not selected, delete the previous character,
            // if the text is selected, delete the characters corresponding
            // to the selection range
            if SelectStartPos == 0 AND SelectEndPos == 0
               this.enteredPassword = ;
                  stuff(this.enteredPassword, nPosition - 1, 1, "")
            else
               this.enteredPassword = ;
                  stuff(this.enteredPassword, SelectStartPos + 1, ;
                  SelectEndPos - SelectStartPos, "")
            endif
         case nChar = DELETE_KEY
            // if the text is not selected, delete the current character,
            // if the text is selected, delete the characters corresponding
            // to the selection range
            if SelectStartPos == 0 AND SelectEndPos == 0
               this.enteredPassword = ;
                  stuff(this.enteredPassword, nPosition, 1, "")
            else
               this.enteredPassword = ;
                  stuff(this.enteredPassword, SelectStartPos + 1, ;
                  SelectEndPos - SelectStartPos, "")
            endif
         case nChar = ENTER_KEY
            if this.captureEnter AND this.enteredPassword.length = 0
		         msgbox( "You must specify a Password!","ALERT!", 48 )
   		      this.setFocus()
      		endif
         otherwise
            if CLASS::IsValidChar(enteredChar)    // Check if alphanumeric
               // if the text is not selected, insert the character entered,
               // if the text is selected, overwrite the characters
               // corresponding to the selection range with the character
               // entered
               if SelectStartPos == 0 AND SelectEndPos == 0
                  this.enteredPassword = ;
                     stuff(this.enteredPassword, nPosition, 1, enteredChar)
               else
                  this.enteredPassword = ;
                     stuff(this.enteredPassword, SelectStartPos + 1, ;
                     SelectEndPos - SelectStartPos, enteredChar)
               endif
               returnValue = asc( this.maskChar ) // Output camouflage character
            endif
      endcase
   return returnValue


   // -----------------------
   Procedure IsValidChar(char)

      // Make sure entered key is alphanumeric
      private returnValue

      do case
         case isalpha(char)                        // Letter?
            returnValue = true
         case char >= "0" and char <= "9"          // Digit?
            returnValue = true
         otherwise                                 // Invalid?
            returnValue = false
      endcase

   return returnValue

   // ----------------
   Procedure LostFocus
      // What had focus? The following statement assumes
      // that the form is using "CANCELPUSHBUTTON" in
      // the TestPass.wfm -- if you are using a different
      // pushbutton class, either rename it, or change
      // the source below (but of course be warned, updates
      // to the dUFLP will overwrite what you're doing)
      if this.parent.activeControl.Name == "CANCELPUSHBUTTON"
         this.stopTrying := true
         return
      endif

      // this should (hopefully) only happen if
      // the user clicks the 'x' in the upper right
      // corner ...
      if this.parent.activeControl.Name == this.Name
         this.stopTrying := true
         return
      endif

      // Deal with retries ...
      if (this.numTriesAllowed == 0 ) OR;
         ( NOT this.validCheckRequired )
         return
      endif

      // check for length -- added, KJM, August 21, 2001
      if this.minLength > 0
         if len( this.enteredPassword ) < this.minLength
            msgbox( "Password must be at least "+this.minLength+" characters.",;
                    "Try again!", 16 )
            this.setFocus()
            return
         endif
      endif

      // if the password is incorrect:
      if not this.IsEnteredPasswordOk() 
         // check to see how many times we've tried it:
         if this.numTries < this.numTriesAllowed and ;
            msgbox( "Invalid password, try again?", "Invalid Password", 20 ) == 6
            // If the number of tries allowed has not been
            // reached, back to the password:
            if this.numTries < this.numTriesAllowed
               this.setFocus()
            else  // number of tries allowed has been reached:
               // used with form to deal with
               // not allowing user to continue
               this.stopTrying = true
             endif // numtries < allowed
         endif // numtries <= allowed

      endif // not ok

endclass
// End of class: PasswordEntry

/*
    End of File: PASSWORD.CC
*/


