// Metric: 0 - Chars only
/* 
   TextFieldSelector.cc
   Rich www.autotraker.com

   This is a CC that allows you to select text in a editor control by
   double clicking, highlighting with mouse, keyboard or by way of 
   spinboxes. Text string and position information is then returned by
   pushing the button. 

   Note that this is a container object with an editor, some spinboxes,
   a checkbox, some textlabel controls, and a pushbutton. 

   This is really meant as sample code. To use in a real-world
   application, you would probably want to extract some of this,
   and set properties as you needed, perhaps using just the 
   spinboxes with the editor control. 

   See TextFieldSelector2 below the first sample class given. 
   There are comments on usage of that version below ...
*/

class TEXTFIELDSELECTOR(parentObj) of CONTAINER(parentObj) custom
   with (this)
      left = 1.0
      top = 0.5
      width = 70.5
      height = 5.1
      borderStyle = 3	// None
   endwith

   this.EDITORFIELD = new EDITOR(this)
   with (this.EDITORFIELD)
      onGotFocus = class::EDITORFIELD_ONGOTFOCUS
      onLostFocus = class::EDITORFIELD_ONLOSTFOCUS
      onLeftDblClick = class::EDITORFIELD_ONLEFTDBLCLICK
      onLeftMouseDown = class::EDITORFIELD_ONLEFTMOUSEDOWN
      onLeftMouseUp = class::EDITORFIELD_ONLEFTMOUSEUP
      height = 1.05
      left = 0.3333
      top = 0.1
      width = 69.0
      value = "TextFieldSelector: This is test text to see how this works"
      wrap = false
      scrollBar = 3	// Disabled
      evalTags = false
      popupEnable = false
   endwith

   this.STARTSPIN = new SPINBOX(this)
   with (this.STARTSPIN)
      onChange = class::STARTSPIN_ONCHANGE
      height = 1.0
      left = 6.8333
      top = 3.95
      width = 8.0
      rangeMax = 100
      rangeMin = 1
      value = 1
      rangeRequired = true
   endwith

   this.FIELDWIDTH = new SPINBOX(this)
   with (this.FIELDWIDTH)
      onChange = class::FIELDWIDTH_ONCHANGE
      height = 1.0
      left = 23.8333
      top = 3.95
      width = 8.0
      rangeMax = 100
      rangeMin = 0
      value = 0
      rangeRequired = true
   endwith

   this.ENDSPIN = new SPINBOX(this)
   with (this.ENDSPIN)
      onChange = class::ENDSPIN_ONCHANGE
      height = 1.0
      left = 38.8333
      top = 3.95
      width = 8.0
      rangeMax = 100
      rangeMin = 1
      value = 0
      rangeRequired = true
   endwith

   this.TEXTLABEL1 = new TEXTLABEL(this)
   with (this.TEXTLABEL1)
      height = 0.9
      left = -0.1667
      top = 4.1
      width = 6.8333
      text = "Start"
      alignHorizontal = 2	// Right
   endwith

   this.TEXTLABEL2 = new TEXTLABEL(this)
   with (this.TEXTLABEL2)
      height = 0.9
      left = 16.1667
      top = 4.1
      width = 6.8333
      text = "Length"
      alignHorizontal = 2	// Right
   endwith

   this.TEXTLABEL3 = new TEXTLABEL(this)
   with (this.TEXTLABEL3)
      height = 0.9
      left = 31.6667
      top = 4.1
      width = 6.8333
      text = "End"
      alignHorizontal = 2	// Right
   endwith

   this.TEXTLABEL4 = new TEXTLABEL(this)
   with (this.TEXTLABEL4)
      height = 0.9
      left = 0.1667
      top = 1.85
      width = 11.3333
      text = "Selection"
   endwith

   this.SELECTFIELD = new ENTRYFIELD(this)
   with (this.SELECTFIELD)
      when = {;return false}
      height = 0.9
      width = 46.8333
      left = 2.1667
      top = 2.7
      value = ""
   endwith

   this.DOUBLECLICKBOX = new CHECKBOX(this)
   with (this.DOUBLECLICKBOX)
      height = 1.0
      left = 18.0
      top = 1.4
      width = 30.6667
      text = "Use double click for selection"
   endwith

   this.SELECTBUTTON = new PUSHBUTTON(this)
   with (this.SELECTBUTTON)
      onClick = class::SELECTBUTTON_ONCLICK
      height = 1.2
      left = 52.1667
      top = 1.95
      width = 17.5
      text = "Define"
   endwith

   function updateControlsFromEditor
      this.EDITORFIELD.copy()
      this.SELECTFIELD.value = ""
      this.SELECTFIELD.paste() 

      // find where in text this string is
      local lenStr,nStart,testStr
      lenStr = len(this.SELECTFIELD.value)
      nStart = 1
      do while true
         testStr = substr(this.EDITORFIELD.value,nStart,lenStr)
         if testStr == this.SELECTFIELD.value
            exit
         endif
         nStart ++
      enddo
      this.STARTSPIN.value := nStart
      this.FIELDWIDTH.value := lenStr
      this.ENDSPIN.value := nStart + lenStr
      return

   function updateEditorFromControls
      this.EDITORFIELD.columnNo = this.STARTSPIN.value
      local nStart
      for nStart = 1 to this.FIELDWIDTH.value
         this.EDITORFIELD.keyboard("{-501}") // rightarrow
      endfor
      return

   function arrowFired
      parameters cDirection
      if cDirection = "rightarrow"
         this.EDITORFIELD.keyboard("{-501}")
      else // left arrow
         this.EDITORFIELD.keyboard("{-500}")
      endif
      this.updateControlsFromEditor()
      return

   function EDITORFIELD_onGotFocus
      _app.currentForm = this.parent
      on key label Shift+rightarrow _app.currentForm.arrowFired("rightarrow")
      on key label Shift+leftarrow _app.currentForm.arrowFired("leftarrow")
      return

   function EDITORFIELD_onLostFocus
      on key label Shift+rightarrow
      on key label Shift+leftarrow
      _app.currentForm = NULL
      return

   function EDITORFIELD_onLeftDblClick(flags, col, row)
      if this.parent.DOUBLECLICKBOX.value
         this.parent.updateControlsFromEditor()
      endif
      return

   function EDITORFIELD_onLeftMouseDown(flags, col, row)
      if this.parent.DOUBLECLICKBOX.value
         return
      endif
      this.parent.STARTSPIN.value := this.parent.EDITORFIELD.columnNo
      this.parent.STARTSPIN.onChange()
      return

   function EDITORFIELD_onLeftMouseUp(flags, col, row)
      if this.parent.DOUBLECLICKBOX.value
         return
      endif
      this.parent.ENDSPIN.value := this.parent.EDITORFIELD.columnNo
      if this.parent.STARTSPIN.value > this.parent.ENDSPIN.value
         nStart = this.parent.STARTSPIN.value
         nEnd   = this.parent.ENDSPIN.value
         this.parent.STARTSPIN.value := nEnd
         this.parent.ENDSPIN.value   := nStart
      endif
      this.parent.ENDSPIN.onChange()
      return

   function ENDSPIN_onChange
      this.parent.ENDSPIN.rangeMax = len(this.parent.EDITORFIELD.value) + 1
      if this.parent.ENDSPIN.value < this.parent.STARTSPIN.value
         this.parent.ENDSPIN.value := this.parent.STARTSPIN.value
      endif
      this.parent.FIELDWIDTH.value := this.value - this.parent.STARTSPIN.value
      this.parent.SELECTFIELD.value = substr(this.parent.EDITORFIELD.value,this.parent.STARTSPIN.value,this.parent.FIELDWIDTH.value)
      this.parent.updateEditorFromControls()
      return

   function FIELDWIDTH_onChange
      this.parent.FIELDWIDTH.rangeMax = len(this.parent.EDITORFIELD.value) - this.parent.STARTSPIN.value + 1
      this.parent.SELECTFIELD.value = substr(this.parent.EDITORFIELD.value,this.parent.STARTSPIN.value,this.parent.FIELDWIDTH.value)
      this.parent.ENDSPIN.value = this.parent.STARTSPIN.value + this.parent.FIELDWIDTH.value
      this.parent.updateEditorFromControls()
      return

   function STARTSPIN_onChange
      this.parent.STARTSPIN.rangeMax = len(this.parent.EDITORFIELD.value)
      if this.parent.ENDSPIN.value < this.parent.STARTSPIN.value
         this.parent.ENDSPIN.value := this.parent.STARTSPIN.value
      endif
      this.parent.FIELDWIDTH.value := this.parent.ENDSPIN.value - this.parent.STARTSPIN.value 
      this.parent.SELECTFIELD.value = substr(this.parent.EDITORFIELD.value,this.parent.STARTSPIN.value,this.parent.FIELDWIDTH.value)
      this.parent.updateEditorFromControls()
      return

   function SELECTBUTTON_onClick
      // store the information on this field and place the cursor in the 
      // right place to start selecting the next field
      if this.parent.FIELDWIDTH.value > 0
         thisFieldStart = this.parent.STARTSPIN.value
         thisFieldLength = this.parent.FIELDWIDTH.value
         thisFieldValue = substr(this.parent.EDITORFIELD.value,this.parent.STARTSPIN.value,this.parent.FIELDWIDTH.value)

         this.parent.EDITORFIELD.columnNo = this.parent.STARTSPIN.value + this.parent.FIELDWIDTH.value
         this.parent.STARTSPIN.value = this.parent.EDITORFIELD.columnNo
         this.parent.ENDSPIN.value = this.parent.EDITORFIELD.columnNo
         this.parent.FIELDWIDTH.value = 0

         msgbox("Start:"+thisFieldStart+" Length:"+thisFieldLength+chr(13)+;
                "Value:"+ thisFieldValue)

         this.parent.EDITORFIELD.setfocus()
      endif
      return
endclass

// -------------------------------------------------------------------
/* 
   TextFieldSelector.cc
   TextFieldSelector2 -- see notes below
   Rich www.autotraker.com

   This is a CC that allows you to select text in a editor control by
   double clicking, highlighting with mouse, keyboard or by way of 
   spinboxes. Text string and position information is then returned by
   pushing the button. 

   This version was modified by Ken Mayer, to remove some controls
   that wouldn't normally be in a control that you would use in a
   real-world application. This was modified as little as possible,
   and all the functionality is still what Rich put in there.

   Usage:
      Set procedure to :dUFLP:TextFieldSelector.cc additive
      * In the form designer, drag TextFieldSelector2 control 
        to the design surface. 
      * Datalink the EditorField control
      * If you need to programmatically access what the user
        has selected, you can do so by code like:
            cSelected = form.TextFieldSelector21.SelectField.value
        SelectField is an entryfield that is not visible on the form,
        but the value property is updated every time the user selects
        something in the editor control.
      * The container has a custom property called doubleClick --
        this defaults to true, if you don't want the user to 
        be able to double-click on the editor to select text,
        set this property to false.
*/

class TEXTFIELDSELECTOR2(parentObj) of CONTAINER(parentObj) custom
   with (this)
      left = 1.0
      top = 0.5
      width = 70.4286
      height = 4.0455
      borderStyle = 3	// None
   endwith
   // custom property of the control, used to 
   // allow the user to double-click and select
   // the current word. If you want this disabled
   // set this property to false.
   this.DoubleClick = true

   this.EDITORFIELD = new EDITOR(this)
   with (this.EDITORFIELD)
      onGotFocus = class::EDITORFIELD_ONGOTFOCUS
      onLostFocus = class::EDITORFIELD_ONLOSTFOCUS
      onLeftDblClick = class::EDITORFIELD_ONLEFTDBLCLICK
      onLeftMouseDown = class::EDITORFIELD_ONLEFTMOUSEDOWN
      onLeftMouseUp = class::EDITORFIELD_ONLEFTMOUSEUP
      height = 2.4545
      scrollBar = 2	// Auto
      left = 0.3333
      top = 0.1
      width = 69.0
      value = "TextFieldSelector: This is test text to see how this works"
      wrap = false
      evalTags = false
      popupEnable = false
   endwith

   this.STARTSPIN = new SPINBOX(this)
   with (this.STARTSPIN)
      onChange = class::STARTSPIN_ONCHANGE
      height = 1.0
      left = 6.8333
      top = 2.7273
      width = 8.0
      rangeMax = 100
      rangeMin = 1
      value = 1
      rangeRequired = true
   endwith

   this.FIELDWIDTH = new SPINBOX(this)
   with (this.FIELDWIDTH)
      onChange = class::FIELDWIDTH_ONCHANGE
      height = 1.0
      left = 23.8333
      top = 2.7273
      width = 8.0
      rangeMax = 100
      rangeMin = 0
      value = 0
      rangeRequired = true
   endwith

   this.ENDSPIN = new SPINBOX(this)
   with (this.ENDSPIN)
      onChange = class::ENDSPIN_ONCHANGE
      height = 1.0
      left = 38.8333
      top = 2.7273
      width = 8.0
      rangeMax = 100
      rangeMin = 1
      value = 0
      rangeRequired = true
   endwith

   this.TEXTLABEL1 = new TEXTLABEL(this)
   with (this.TEXTLABEL1)
      height = 0.7727
      left = 0.7143
      top = 2.8636
      width = 4.8571
      text = "Start"
      alignHorizontal = 2	// Right
   endwith

   this.TEXTLABEL2 = new TEXTLABEL(this)
   with (this.TEXTLABEL2)
      height = 0.7727
      top = 2.8636
      left = 16.1667
      width = 6.8333
      text = "Length"
      alignHorizontal = 2	// Right
   endwith

   this.TEXTLABEL3 = new TEXTLABEL(this)
   with (this.TEXTLABEL3)
      height = 0.7727
      left = 33.1429
      top = 2.8636
      width = 4.2857
      text = "End"
      alignHorizontal = 2	// Right
   endwith

   this.SELECTFIELD = new ENTRYFIELD(this)
   with (this.SELECTFIELD)
      when = {;return false}
      height = 0.9
      width = 46.8333
      left = 4
      top = 6.2273
      value = ""
      visible = false
   endwith

   function updateControlsFromEditor
      this.EDITORFIELD.copy()
      this.SELECTFIELD.value = ""
      this.SELECTFIELD.paste() 

      // find where in text this string is
      local lenStr,nStart,testStr
      lenStr = len(this.SELECTFIELD.value)
      nStart = 1
      do while true
         testStr = substr(this.EDITORFIELD.value,nStart,lenStr)
         if testStr == this.SELECTFIELD.value
            exit
         endif
         nStart ++
      enddo
      this.STARTSPIN.value := nStart
      this.FIELDWIDTH.value := lenStr
      this.ENDSPIN.value := nStart + lenStr
      return

   function updateEditorFromControls
      this.EDITORFIELD.columnNo = this.STARTSPIN.value
      local nStart
      for nStart = 1 to this.FIELDWIDTH.value
         this.EDITORFIELD.keyboard("{-501}") // rightarrow
      endfor
      return

   function arrowFired
      parameters cDirection
      if cDirection = "rightarrow"
         this.EDITORFIELD.keyboard("{-501}")
      else // left arrow
         this.EDITORFIELD.keyboard("{-500}")
      endif
      this.updateControlsFromEditor()
      return

   function EDITORFIELD_onGotFocus
      _app.currentForm = this.parent
      on key label Shift+rightarrow _app.currentForm.arrowFired("rightarrow")
      on key label Shift+leftarrow _app.currentForm.arrowFired("leftarrow")
      return

   function EDITORFIELD_onLostFocus
      on key label Shift+rightarrow
      on key label Shift+leftarrow
      _app.currentForm = NULL
      return

   function EDITORFIELD_onLeftDblClick(flags, col, row)
      if this.parent.DOUBLECLICK
         this.parent.updateControlsFromEditor()
      endif
      return

   function EDITORFIELD_onLeftMouseDown(flags, col, row)
      if this.parent.DOUBLECLICK
         return
      endif
      this.parent.STARTSPIN.value := this.parent.EDITORFIELD.columnNo
      this.parent.STARTSPIN.onChange()
      return

   function EDITORFIELD_onLeftMouseUp(flags, col, row)
      if this.parent.DOUBLECLICK
         return
      endif
      this.parent.ENDSPIN.value := this.parent.EDITORFIELD.columnNo
      if this.parent.STARTSPIN.value > this.parent.ENDSPIN.value
         nStart = this.parent.STARTSPIN.value
         nEnd   = this.parent.ENDSPIN.value
         this.parent.STARTSPIN.value := nEnd
         this.parent.ENDSPIN.value   := nStart
      endif
      this.parent.ENDSPIN.onChange()
      return

   function ENDSPIN_onChange
      this.parent.ENDSPIN.rangeMax = len(this.parent.EDITORFIELD.value) + 1
      if this.parent.ENDSPIN.value < this.parent.STARTSPIN.value
         this.parent.ENDSPIN.value := this.parent.STARTSPIN.value
      endif
      this.parent.FIELDWIDTH.value := this.value - this.parent.STARTSPIN.value
      this.parent.SELECTFIELD.value = substr(this.parent.EDITORFIELD.value,this.parent.STARTSPIN.value,this.parent.FIELDWIDTH.value)
      this.parent.updateEditorFromControls()
      return

   function FIELDWIDTH_onChange
      this.parent.FIELDWIDTH.rangeMax = len(this.parent.EDITORFIELD.value) - this.parent.STARTSPIN.value + 1
      this.parent.SELECTFIELD.value = substr(this.parent.EDITORFIELD.value,this.parent.STARTSPIN.value,this.parent.FIELDWIDTH.value)
      this.parent.ENDSPIN.value = this.parent.STARTSPIN.value + this.parent.FIELDWIDTH.value
      this.parent.updateEditorFromControls()
      return

   function STARTSPIN_onChange
      this.parent.STARTSPIN.rangeMax = len(this.parent.EDITORFIELD.value)
      if this.parent.ENDSPIN.value < this.parent.STARTSPIN.value
         this.parent.ENDSPIN.value := this.parent.STARTSPIN.value
      endif
      this.parent.FIELDWIDTH.value := this.parent.ENDSPIN.value - this.parent.STARTSPIN.value 
      this.parent.SELECTFIELD.value = substr(this.parent.EDITORFIELD.value,this.parent.STARTSPIN.value,this.parent.FIELDWIDTH.value)
      this.parent.updateEditorFromControls()
      return

   function SELECTBUTTON_onClick
      // store the information on this field and place the cursor in the 
      // right place to start selecting the next field
      if this.parent.FIELDWIDTH.value > 0
         thisFieldStart = this.parent.STARTSPIN.value
         thisFieldLength = this.parent.FIELDWIDTH.value
         thisFieldValue = substr(this.parent.EDITORFIELD.value,this.parent.STARTSPIN.value,this.parent.FIELDWIDTH.value)

         this.parent.EDITORFIELD.columnNo = this.parent.STARTSPIN.value + this.parent.FIELDWIDTH.value
         this.parent.STARTSPIN.value = this.parent.EDITORFIELD.columnNo
         this.parent.ENDSPIN.value = this.parent.EDITORFIELD.columnNo
         this.parent.FIELDWIDTH.value = 0

         msgbox("Start:"+thisFieldStart+" Length:"+thisFieldLength+chr(13)+;
                "Value:"+ thisFieldValue)

         this.parent.EDITORFIELD.setfocus()
      endif
      return
endclass

