/*
   kmScheduler.h
   This is a set of definitions of strings used in the forms of
   the Calendar/Scheduler created by Ken Mayer. The idea is to
   place all character strings that are displayed for the user
   in this file as defined constants (using the preprocessor
   command: #DEFINE). In the code for the four different forms,
   anywhere that a character string is displayed, it should be
   replaced by the appropriate constant in the header file.
   This will have the effect of making it possible to have
   folk who know other languages as well as English translate
   the strings, and make this more non-English capable. 

   When  using these, each of the following four forms will need
   to have the command:
       #INCLUDE <kmScheduler_English.h>
   changed to use the appropriate header file, such as:
       #INCLUDE <kmScheduler_German.h>

   The forms are:
       kmScheduler.wfm
       kmSchedEvent.wfm
       km24Time.wfm
       kmSmallCal.wfm

   For those doing the translation, if any of the character strings
   require the use of upper ANSI characters, such as u with an umlaut
   (just an example), rather than typing that in, it may be a good
   idea to use chr( 252 ), as in: "abcde"+chr(252)+"test".
*/ 

// ------------------------------------------------------------------
// For the main calendar form: kmScheduler.wfm
// ------------------------------------------------------------------
// Form's text property:
#DEFINE SCHEDFORMTEXT             "Kalender/Agenda"

// UI -- User Interface -- speedtips for controls:
#DEFINE MONTHBUTTONTEXT           "Maand"
#DEFINE NEXTMONTHBUTTONSPEEDTIP   "Volgende maand"
#DEFINE PREVMONTHBUTTONSPEEDTIP   "Vorige maand"
#DEFINE YEARBUTTONTEXT            "Jaar"
#DEFINE NEXTYEARBUTTONSPEEDTIP    "Volgend jaar"
#DEFINE PREVYEARBUTTONSPEEDTIP    "Vorig jaar"
#DEFINE TODAYBUTTONTEXT           "Vandaag"
#DEFINE TODAYBUTTONSPEEDTIP       "Ga naar huidige datum"
#DEFINE WEEKVIEWBUTTONTEXT        "Weekoverzicht"
#DEFINE WEEKVIEWBUTTONSPEEDTIP    "Ga naar weekoverzicht"
#DEFINE WEEKBUTTONTEXT            "Week"
#DEFINE NEXTWEEKBUTTONSPEEDTIP    "Volgende week"
#DEFINE PREVWEEKBUTTONSPEEDTIP    "Vorige week"
#DEFINE MONTHVIEWBUTTONTEXT       "Maandoverzicht"
#DEFINE MONTHVIEWBUTTONSPEEDTIP   "Ga naar maandoverzicht"
#DEFINE MOREBUTTONUPSPEEDTIP      "Ga naar vorige vijf items"
#DEFINE MOREBUTTONDOWNSPEEDTIP    "Ga naar volgende vijf items"

// canOpen event:
#DEFINE OPENFORMERROR             "Kan het formulier niet openen"
#DEFINE DATEEXOPENERROR           "Kan :dUFLP:DateEx.cc niet openen!"
#DEFINE HOLIDAYOPENERROR          "Kan :dUFLP:Holiday.cc niet openen!"
#DEFINE BDEALIASOPENERROR         "Kan :dUFLP:BDEAlias.cc niet openen!"
// note the trailing space before the quote -- keep it!
#DEFINE CREATEALIASERROR          "Kan de BDE-alias niet maken: "
#DEFINE CREATETABLEERROR          "Kan de tabel niet maken: "
#DEFINE TABLELEVELERROR           "Bestandsniveau moet 7 zijn - wijzig dit in de BDE configuratie a.u.b.!"
#DEFINE CREATEINDEXESERROR        "Kan geen tabel indexen maken: "
// Holiday names:
#DEFINE ADVENTNAME                "Advent"
#DEFINE EASTERNAME                "Pasen"
#DEFINE NEWYEARNAME               "Nieuwjaarsdag"
#DEFINE VALENTINENAME             "Valentijnsdag"
#DEFINE STPATRICKNAME             "Sint-Patricks dag"
#DEFINE APRILFOOLNAME             "Eerste april"
#DEFINE HALLOWEENNAME             "Halloween"
#DEFINE CHRISTMASNAME             "Kerstmis"
#DEFINE NEWYEAREVENAME            "Nieuwjaarsavond"


// ------------------------------------------------------------------
// kmSchedEvent.wfm
// ------------------------------------------------------------------
// UI Interface:
// Form text:
#DEFINE SCHEDEVENTFORMTEXT        "Details individuele gebeurtenis"
// Tabbox1 dataSource -- don't translate the word "ARRAY":
#DEFINE TABBOXDATASOURCE          [ARRAY {"Huidige gebeurtenis","De gebeurtenissen van de dag"}]
// Title text:
#DEFINE TITLETEXTEVENTS           "Gebeurtenis(sen)"
// Grid column headings:
#DEFINE GRIDCOLUMN1HEADING        "Tijd"
#DEFINE GRIDCOLUMN2HEADING        "Gebeurtenis"
#DEFINE GRIDCOLUMN3HEADING        "Herhaal"
// Text/TextLabel controls on form:
#DEFINE EVENTTITLETEXT            "Titel gebeurtenis:"
#DEFINE DETAILTEXT                "Detail:"
#DEFINE LOCATIONTEXT              "Locatie:"
#DEFINE DATETEXT                  "Datum:"
#DEFINE STARTTIMETEXT             "Starttijd:"
#DEFINE ENDTIMETEXT               "Eindtijd:"
#DEFINE REPEATTYPETEXT            "Herhaal type:"
#DEFINE REPEATEVENTSTEXT          "Herhaalde gebeurtenissen"
#DEFINE EVENTNOTREPEATTEXT        "Deze gebeurtenis herhaald niet"
#DEFINE STATUSTEXT                "Status:"
// Note the HTML formatting here -- leave those, but move as appropriate:
#DEFINE INFOTEXT                  "<b>De datum</b> is noodzakelijk, evenals de standaard waarden "+;
                                  "voor de geselecteerde datum van de gebeurtenis. "+;
                                  "<b>Starttijd</b> en <b>eindtijd</b> "+;
                                  "zijn nodig, met uitzondering voor een gebeurtenis die een 'volledige dag' "+;
                                  "in beslag neemt.  Tijd moet in het 24-uur formaat."
// Checkbox:
#DEFINE ALLDAYCHECKTEXT           "Volledige dag?"
// Buttons:
#DEFINE CHANGEDATEBUTTONSPEEDTIP  "Wijzig datum"
#DEFINE STARTTIMESPEEDTIP         "Wijzig starttijd"
#DEFINE ENDTIMESPEEDTIP           "Wijzig eindtijd"
// Comboboxes:
#DEFINE REPEATTYPE1               "Geen"
#DEFINE REPEATTYPE2               "Dag"
#DEFINE REPEATTYPE3               "Week"
#DEFINE REPEATTYPE4               "Maand"
#DEFINE REPEATTYPE5               "Jaar"
#DEFINE STATUSCOMBO1              "Open"
#DEFINE STATUSCOMBO2              "Volledig"
// On 'repeat' containers:
#DEFINE EVERYTEXT                 "Alles"
#DEFINE ENDONTEXT                 "Einde op:"
#DEFINE REPEATSPINERROR           "Reeks van 1 tot 99"
#DEFINE ENDDATEBUTTONSPEEDTIP     "Wijzig datum"
#DEFINE DAYSTEXT                  "Dag(en)"
#DEFINE WEEKSTEXT                 "Week (weken)"
#DEFINE MONTHSTEXT                "Maand(en)"
#DEFINE YEARSTEXT                 "Jaar (jaren)"
#DEFINE DESCRIPTTEXT              "De ste/de van iedere maand"
#DEFINE REPEATONTEXT              "Herhaal op:"
// Delete method:
#DEFINE DELETEQUESTIONTEXT        "Wilt u enkel de huidige gebeurtenis verwijderen (ja), of alle gebeurtenissen (neen)?"
#DEFINE REPEATINGEVENTTITLE       "Herhaalde gebeurtenis"
#DEFINE UNEXPECTEDERRORMSG        "Onverwachte fout: "
#DEFINE ERRORTITLE                "Fout"
// Save Method:
#DEFINE SAVEREPEATQUESTIONTEXT    "Wilt u deze wijzigingen enkel op de huidige gebeurtenis toepassen (ja), of op alle gebeurtenissen (neen)?"
#DEFINE TITLEREQUIREDTEXT         "Titel gebeurtenis is vereist!"
#DEFINE CANNOTSAVETITLE           "Kan niet opslaan!"
#DEFINE EVENTREQUIREDTEXT         "Datum gebeurtenis is vereist!"
#DEFINE STARTTIMEREQUIREDTEXT     "Starttijd gebeurtenis is vereist!"
#DEFINE ENDTIMEREQUIREDTEXT       "Eindtijd gebeurtenis is vereist!"
#DEFINE ENDDATEREQUIREDTEXT       "Een einddatum is vereist voor een gebeurtenis die herhaald wordt!"
#DEFINE DAYOFWEEKREQUIREDTEXT     "Er moet ten minste een dag van de week geselecteerd worden voor een gebeurtenis die wekelijks herhaald wordt!"
// Week Day checkbox change:
#DEFINE WEEKDAYERROR1             "De huidige geselecteerde datum: "
#DEFINE WEEKDAYERROR2             "is een "
#DEFINE WEEKDAYERROR3             ", u kunt dit niet de-selecteren."
#DEFINE WEEKDAYERRORTITLE         "Dit is niet mogelijk"
// TimeEntryfield class:
#DEFINE TIMEENTRYSPEEDTIP         "Geef de tijd in (24-uur formaat)"
#DEFINE TIMEENTRYHOURERRORTEXT    "Geef een waarde in van 0 tot 23"
#DEFINE TIMEENTRYHOURERRORTITLE   "Ongeldig uur"
#DEFINE TIMEENTRYMINUTEERRORTEXT  "Geef een waarde in van 0 tot 59"
#DEFINE TIMEENTRYMINUTEERRORTITLE "Ongeldige minuten"

// ------------------------------------------------------------------
// km24Time.wfm
// ------------------------------------------------------------------
#DEFINE TIMEFORMTEXT              "Selecteer tijd"
#DEFINE HOURTEXT                  "Uur"
#DEFINE MINUTETEXT                "Minuut"
#DEFINE TIMEFORMDESCRIPTIONTEXT   "Tijd is in het 24-uur formaat, voor "+;
                                  "ieder uur na de middag, voeg er 12 bij.  Bv., "+;
                                  "2:00 pm wordt 14:00. "+;
                                  "Middernacht is '00'."
#DEFINE OKBUTTONTEXT              "OK"
#DEFINE TIMEOKBUTTONSPEEDTIP      "Accepteer huidige tijd"
#DEFINE CANCELBUTTONTEXT          "Annuleer"
#DEFINE TIMECANCELBUTTONSPEEDTIP  "Ga terug naar vorig geselecteerde waarde"

// ------------------------------------------------------------------
// kmSmallCal.wfm
// ------------------------------------------------------------------
#DEFINE CALFORMTEXT               "Selecteer datum"
#DEFINE DATEOKBUTTONSPEEDTIP      "Accepteer huidige datum"

// ------------------------------------------------------------------
// :dUFLP:kenCal3.cc (on kmSmallCal.wfm)
// ------------------------------------------------------------------
#DEFINE KCYEARMINUS10BUTTONSPEEDTIP "10 jaar terug"
#DEFINE KCYEARPLUS10BUTTONSPEEDTIP  "10 jaar vooruit"
#DEFINE KCPREVYEARBUTTONSPEEDTIP    "Vorig jaar"
#DEFINE KCNEXTYEARBUTTONSPEEDTIP    "Volgend jaar"
#DEFINE KCMONTHMINUS3BUTTONSPEEDTIP "3 maanden terug"
#DEFINE KCMONTHPLUS3BUTTONSPEEDTIP  "3 maanden vooruit"
#DEFINE KCPREVMONTHBUTTONSPEEDTIP   "Vorige maand"
#DEFINE KCNEXTMONTHBUTTONSPEEDTIP   "Volgende maand"
#DEFINE KCTODAYBUTTONTEXT           "Vandaag"
#DEFINE KCTODAYBUTTONSPEEDTIP       "Ga naar huidige datum"

// ------------------------------------------------------------------
// kmSchedColor.wfm
// ------------------------------------------------------------------
// This one is actually in kmScheduler.wfm
#DEFINE CHANGECOLORBUTTONSPEEDTIP     "Wijzig de knopkleuren"
// The rest are in kmSchedColor.wfm
#DEFINE COLORFORMTITLE                "Wijzig de kleuren"
#DEFINE HOLIDAYBUTTONTEXT             "Feestdag"
#DEFINE CURRENTDAYBUTTONTEXT          "Huidig geselecteerde dag"
#DEFINE DEFAULTDAYBUTTONTEXT          "Standaard"
#DEFINE TEXTCOLORBUTTONTEXT           "T"
#DEFINE BACKGROUNDCOLORBUTTONTEXT     "A"
#DEFINE TEXTCOLORBUTTONSPEEDTIP       "Wijzig de tekstkleur"
#DEFINE BACKGROUNDCOLORBUTTONSPEEDTIP "Wijzig de achtergrondkleur"



