// -- File: RECNOQRY.CC

CLASS RecNoQuery of QUERY Custom
/* ----------------------------------------------------------------------
   Description:
 
     A custom control written by Romain Strieff. Through the use
     of BDE API calls, there are four additional methods which
     may be useful for a developer, particularly one used to the
     older XBase DML commands:

 
   Programmers: 

       Romain Strieff 
       Ken Mayer (some very minor stuff here)
 
    History: 
 
       01/07/1998 -- First appearance of this on the Borland
                     newsgroups (posted by Romain)
       03/26/1998 -- Updated ...
       10/29/1998 -- New posting of this class on newsgroups
 
    Instantiation: 
 
       set procedure to :dUFLP:recnoqry.cc additive
       cMyQuery = new RecNoQry()
       cMyQuery.sql = ...
       cMyQuery.active = true

       (This could be used in a data module, as well ... or any
       place you normally might "drag" a query to a form, report,
       data module ... you can edit source code and change
       the code from "new Query()" to "new RecNoQry()" -- although
       you may want to ensure that the SET PROCEDURE statement
       mentioned above is included somewhere)
 
 
    Methods:
       OnOpen()
         When the query opens set some calls to the BDE API.

       RecNo()
         Returns the current row number (like the xBASE 
         DML function RecNo() )

       RecCount()
         Returns the number of rows in the table currently.

       RecOfTotalString()
         Returns a character string showing current row OF
         Reccount ...

       GotoRecord()
         Use the record number returned by the RecNo() method,
         or if a number is known, use that ...


  Usage Notes:
     Use as a normal query, with the exception that you place
     the name of the custom query in place of "query()":

     q = new RecNoQuery()
     q.sql = "select * from mytable"
     q.active = true


     The extra methods can be used as you might expect:

     ? q.rowset.RecNo()
     ? q.rowset.RecCount()
     ? q.rowset.RecOfTotalString()
     q.rowset.GotoRecord( 13 )
     ? q.rowset.RecNo()

*/

   if type("GetRecNo") # "FP"
     extern cint GetRecNo( cHandle, cInt, cptr, cptr);
       IDAPI32 from "DbiGetRecord"
   endif
  
   if type("GotoRec") # "FP"
      extern cint GotoRec( cHandle, cInt);
        IDAPI32 from "DbiSetToRecordNo"
   endif 
 
   if type("GetRecCount") # "FP"
     extern cint GetRecCount( cHandle, cptr);
       IDAPI32 from "DbiGetRecordCount"
   endif


   Function RecNo
      local c,nRet,h
      nRet=0
      h=this.rowset.handle
      c = space(8)
      if (GetRecNo(h,0,"",c)==0)
         nRet= c.getByte( 4 )+bitlshift( c.getByte( 4+1),4)
      endif 
   return int(nRet)


   Function GoToRec
      Parameters nRec
      local lRet,h
      h=this.rowset.handle
      lRet=.f.
      if pcount()=1 and type("nRec")="N"
         if (GoToRec(h,nRec)==0)
            lRet=.t.
         endif 
      endif 
   return lRet


   Function RecCount
      local c,nRet,h
      nRet=0
      h=this.rowset.handle
      c=space(4)
      if (GetRecCount(h,c)==0)
         nRet=c.getByte(0)+ bitlshift( c.getByte(0+1),0)
      endif 
   return int(nRet)

   Function RecOfTotalString
   return str(this.recno()).lefttrim() +"/"+ ;
          str(this.reccount()).lefttrim()

endclass

// End Of Class: RecNoQuery



